unit HomeBrewProfiler;

interface

type
  IHBProfiler = interface ['{35BF43BC-1469-4F35-8DCC-013997E4D2C1}']
    procedure Start;
    function  Stop: string;
    procedure Checkpoint(numCheck: integer);
  end;

var
  Profiler: IHBProfiler;

implementation

uses
  SysUtils,
  WinAPI.MMSystem;

type
  THBProfiler = class(TInterfacedObject, IHBProfiler)
  strict private
    FStartTime: cardinal;
    FLastCheckpoint: cardinal;
    FCheckpoints: array [1..100] of cardinal;
    FUsedCheckpoints: array [1..100] of boolean;
  strict protected
    function  Now: cardinal; inline;
  public
    procedure Start;
    function  Stop: string;
    procedure Checkpoint(numCheck: integer);
  end;

{ THBProfiler }

procedure THBProfiler.Checkpoint(numCheck: integer);
begin
  Inc(FCheckpoints[numCheck], Now - FLastCheckpoint);
  FUsedCheckpoints[numCheck] := true;
  FLastCheckpoint := Now;
end;

function THBProfiler.Now: cardinal;
begin
  Result := timeGetTime;
end;

procedure THBProfiler.Start;
begin
  FStartTime := Now;
  FLastCheckpoint := FStartTime;
end;

function THBProfiler.Stop: string;
var
  iCheckpoint: integer;
begin
  Result := Format('Total execution time: %d ms', [Now - FStartTime]);
  for iCheckpoint := Low(FCheckpoints) to High(FCheckpoints) do
    if FUsedCheckpoints[iCheckpoint] then
      Result := Result + Format('; #%d: %d ms', [iCheckpoint, FCheckpoints[iCheckpoint]]);
end;

initialization
  Profiler := THBProfiler.Create;
  timeBeginPeriod(1);
finalization
  timeEndPeriod(1);
end.
