unit procedureList;

interface

uses
  Classes;

type
  TProcedureList = class;

  TProcedureListEnumerator = record
  strict private
    FProcedureList: TProcedureList;
    FFilter: string;
    FIndex: integer;
  public
    constructor Create(owner: TProcedureList; const filter: string);
    function GetEnumerator: TProcedureListEnumerator;
    function GetCurrent: string;
    function MoveNext: boolean;
    property Current: string read GetCurrent;
  end;

  TProcedureList = class
  strict private
    FList: TStringList;
  private
    function GetItems(index: integer): string;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(const procName: string);
    function Count: integer;
    function Filter(const subStr: string): TProcedureListEnumerator;
    property Items[Index: Integer]: string read GetItems; default;
  end;

implementation

uses
  SysUtils,
  StrUtils;

procedure TProcedureList.Add(const procName: string);
begin
  FList.Add(procName);
end;

function TProcedureList.Count: integer;
begin
  Result := FList.Count;
end;

constructor TProcedureList.Create;
begin
  inherited Create;
  FList := TStringList.Create;
end;

destructor TProcedureList.Destroy;
begin
  FreeAndNil(FList);
  inherited;
end;

function TProcedureList.Filter(const subStr: string): TProcedureListEnumerator;
begin
  Result := TProcedureListEnumerator.Create(Self, subStr);
end;

function TProcedureList.GetItems(index: integer): string;
begin
  Result := FList[index];
end;

{ TProcedureListEnumerator }

constructor TProcedureListEnumerator.Create(owner: TProcedureList; const filter: string);
begin
  FProcedureList := owner;
  FFilter := filter;
  FIndex := -1;
end;

function TProcedureListEnumerator.GetCurrent: string;
begin
  Result := FProcedureList[FIndex];
end;

function TProcedureListEnumerator.GetEnumerator: TProcedureListEnumerator;
begin
  Result := TProcedureListEnumerator.Create(FProcedureList, FFilter);
end;

function TProcedureListEnumerator.MoveNext: boolean;
begin
  Result := false;
  repeat
    Inc(FIndex);
    if FIndex >= FProcedureList.Count then
      Exit;
  until (FFilter = '') or (ContainsText(FProcedureList[FIndex], FFilter));
  Result := true;
end;

end.
