unit procedureListDemo1;

interface

uses
  WinAPI.Windows, WinAPI.Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, ComCtrls,
  procedureList;

type
  TfrmProcedureListDemo = class(TForm)
    inpFilter: TEdit;
    lvProcedureList: TListView;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure inpFilterChange(Sender: TObject);
  private
    FProcedures: TProcedureList;
    procedure AddItem(const methClass, methName: string);
    procedure ReloadView;
    procedure SplitName(const procName: string; var methClass, methName: string);
  end;

var
  frmProcedureListDemo: TfrmProcedureListDemo;

implementation

uses
  procedureListLoader,
  HomeBrewProfiler;

{$R *.dfm}

procedure LoadProcedureList(const fileName: string; procedures: TProcedureList);
var
  s : string;
  sl: TStringList;
begin
  sl := TStringList.Create;
  try
    sl.LoadFromFile(fileName);
    sl.Sorted := true;
    for s in sl do
      procedures.Add(s);
  finally
    FreeAndNil(sl);
  end;
end;

procedure TfrmProcedureListDemo.AddItem(const methClass, methName: string);
var
  listItem: TListItem;
begin
  listItem := lvProcedureList.Items.Add;
  listItem.Caption := methClass;
  listItem.SubItems.Add(methName);
end;

procedure TfrmProcedureListDemo.FormCreate(Sender: TObject);
begin
  FProcedures := TProcedureList.Create;
  LoadProcedureList('..\..\short.dat', FProcedures);
//  LoadProcedureList('..\..\methods.dat', FProcedures);
  ReloadView;
end;

procedure TfrmProcedureListDemo.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FProcedures);
end;

procedure TfrmProcedureListDemo.inpFilterChange(Sender: TObject);
begin
  ReloadView;
end;

procedure TfrmProcedureListDemo.ReloadView;
var
  listItem: TListItem;
  methClass: string;
  methName: string;
  procName: string;
begin
  lvProcedureList.Clear;
  for procName in FProcedures.Filter(inpFilter.Text) do begin
    SplitName(procName, methClass, methName);
    listItem := lvProcedureList.Items.Add;
    listItem.Caption := methClass;
    listItem.SubItems.Add(methName);
  end;
end;

procedure TfrmProcedureListDemo.SplitName(const procName: string; var
    methClass, methName: string);
var
  p: integer;
begin
  p := Pos('.', procName);
  if p = 0 then begin
    methClass := '';
    methName := procName
  end
  else begin
    methClass := Copy(procName, 1, p-1);
    methName := Copy(procName, p+1, Length(procName) - p);
  end;
end;

end.
