unit procedureListDemo2;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, VirtualTrees, StdCtrls,
  procedureList;

type
  TfrmProcedureListDemo = class(TForm)
    inpFilter: TEdit;
    vtProcedureList: TVirtualStringTree;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure inpFilterChange(Sender: TObject);
    procedure vtProcedureListGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column:
      TColumnIndex; TextType: TVSTTextType; var CellText: string);
  private
    FProcedures: TProcedureList;
    FMethodClassList: TStringList;
    FMethodNameList: TStringList;
    procedure ReloadView(const filterStr: string);
    procedure SplitName(const procName: string; var methClass, methName: string);
  end;

var
  frmProcedureListDemo: TfrmProcedureListDemo;

implementation

uses
  procedureListLoader;

{$R *.dfm}

procedure LoadProcedureList(const fileName: string; procedures: TProcedureList);
var
  s : string;
  sl: TStringList;
begin
  sl := TStringList.Create;
  try
    sl.LoadFromFile(fileName);
    for s in sl do
      procedures.Add(s);
  finally
    FreeAndNil(sl);
  end;
end;

{ TfrmProcedureListDemo }

procedure TfrmProcedureListDemo.FormCreate(Sender: TObject);
begin
  FProcedures := TProcedureList.Create;
  FMethodClassList := TStringList.Create;
  FMethodNameList := TStringList.Create;
  LoadProcedureList('..\..\methods.dat', FProcedures);
  ReloadView('');
end;

procedure TfrmProcedureListDemo.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FMethodNameList);
  FreeAndNil(FMethodClassList);
  FreeAndNil(FProcedures);
end;

procedure TfrmProcedureListDemo.inpFilterChange(Sender: TObject);
begin
  ReloadView(inpFilter.Text);
end;

procedure TfrmProcedureListDemo.ReloadView(const filterStr: string);
var
  methClass: string;
  methName: string;
  procName: string;
begin
  vtProcedureList.BeginUpdate;
  try
    vtProcedureList.Clear;
    FMethodClassList.Clear;
    FMethodNameList.Clear;
    for procName in FProcedures.Filter(filterStr) do begin
      SplitName(procName, methClass, methName);
      FMethodClassList.Add(methClass);
      FMethodNameList.Add(methName);
      vtProcedureList.AddChild(nil, pointer(FMethodClassList.Count-1));
    end;
  finally vtProcedureList.EndUpdate; end;
end;

procedure TfrmProcedureListDemo.SplitName(const procName: string; var
    methClass, methName: string);
var
  p: integer;
begin
  p := Pos('.', procName);
  if p = 0 then begin
    methClass := '';
    methName := procName
  end
  else begin
    methClass := Copy(procName, 1, p-1);
    methName := Copy(procName, p+1, Length(procName) - p);
  end;
end;

procedure TfrmProcedureListDemo.vtProcedureListGetText(Sender: TBaseVirtualTree; Node:
  PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
var
  idx: integer;
begin
  idx := PInteger(vtProcedureList.GetNodeData(node))^;
  case Column of
    0: CellText := FMethodClassList[idx];
    1: CellText := FMethodNameList[idx];
  end;
end;

end.
